/**
 * @file opstart.c
 * Start/Stop oprofile
 *
 * @remark Copyright 2007 Openedhand Ltd.
 * @remark Read the file COPYING
 *
 * @author Richard Purdie
 */

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

int main(const int argc, const char* argv[])
{
	const char *enable = "/dev/oprofile/enable";
	const char *lockfile;
	unsigned long dpid;
	struct stat sbuf;
	FILE *lfile, *efile;
	int sig, enb, err;

	if (argc >= 2) {
		printf("Error: Invalid options.\n");
		return 1;
	}

	lockfile = getenv("LOCK_FILE");
	if (!lockfile)
		lockfile = "/var/lib/oprofile/lock";

	/* Add SESSION_DIR support? */

	if (geteuid()) {
		printf("Error: This program must be run as root.\n");
		return 1;
	}

	if (stat(enable, &sbuf)) {
		printf("Error: Could not find /dev/oprofile/enable, the"
		       " kernel module probably isn't loaded.\n");
		printf("This binary only works with 2.6 kernels and oprofile"
		       " must have been initialised with 'opcontrol --start-daemon'.\n");
		return 1;
	}

	if (stat(lockfile, &sbuf)) {
		printf("Error: Could not find lockfile %s.\n", lockfile);
		printf("The oprofile daemon must be running (oprofile must"
		       " have been initialised with 'opcontrol --start-daemon').\n");
		return 1;
	}

	lfile = fopen(lockfile, "r");
	if (!lfile) {
		printf("Error opening lockfile %s.\n", lockfile);
		return 1;
	}

	err = fscanf(lfile, "%lud", (unsigned long *) &dpid);
	if (err != 1) {
		printf("Error reading pid from lockfile %s.\n", lockfile);
		return 1;
	}
	fclose(lfile);

	efile = fopen(enable, "r");
	if (!efile) {
		printf("Error opening %s.\n", enable);
		return 1;
	}

	if (strstr(argv[0], "opstart")) {
		printf("Starting Profiler\n");
		sig = SIGUSR1;
		enb = 1;
	} else if (strstr(argv[0], "opstop")) {
		printf("Stopping Oprofile.\n");
		printf("You need to run 'opcontrol --dump' when the session"
		       " is finished.\n");
		sig = SIGUSR2;
		enb = 0;
	} else {
		printf("Error: Please call as 'opstart' or 'opstop'\n");
		return 1;
	}

	err = kill(dpid, 0);
	if (err) {
		printf("Error sending signal to oprofiled. Stale lockfile"
		       " (%s) ?\n", lockfile);
		return 1;
	}

	fprintf(efile, "%d\n", enb);
	err = kill(dpid, sig);
	if (err) {
		printf("Error sending signal to oprofiled. Stale lockfile"
		       " (%s) ?\n", lockfile);
		return 1;
	}

	return 0;
}

